@@ -152,7 +152,7 @@ Controls::driveUnit(const InputProvider *input)
     }
 
     if (moved != ControlSym::SYM_NONE) {
-        m_moves.push_back(moved);
+        m_moves.append(1, moved);
     }
     return (moved != ControlSym::SYM_NONE);
 }
@@ -280,6 +280,34 @@ Controls::activateSelected(const Cube *occupant)
     }
     return false;
 }
+//-----------------------------------------------------------------
+    void
+Controls::setMoves(const std::string &moves)
+{
+    m_moves = moves;
+    if (!m_moves.empty()) {
+        activateDriven(m_moves[m_moves.size() - 1]);
+    }
+}
+//-----------------------------------------------------------------
+/**
+ * Activate fish driven by given symbol.
+ * @param symbol one of fish symbols
+ * @return true when fish was selected
+ */
+bool
+Controls::activateDriven(char symbol)
+{
+    t_units::iterator end = m_units.end();
+    for (t_units::iterator i = m_units.begin(); i != end; ++i) {
+        if ((*i)->isDrivenBy(symbol)) {
+            m_active = i;
+            m_switch = true;
+            return true;
+        }
+    }
+    return false;
+}
 //-----------------------------------------------------------------
 /**
  * Change active unit.
@@ -305,7 +333,7 @@ Controls::makeMove(char move)
     for (t_units::iterator i = m_units.begin(); i != end; ++i) {
         if ((*i)->driveOrder(move) == move) {
             setActive(i);
-            m_moves.push_back(move);
+            m_moves.append(1, move);
             return true;
         }
     }
@@ -339,5 +367,19 @@ Controls::isPowerful() const
     }
     return result;
 }
+//-----------------------------------------------------------------
+/**
+ * Returns true when active fish is doing a dangerous move.
+ */
+bool
+Controls::isDangerousMove() const
+{
+    bool result = false;
+    if (m_active != m_units.end()) {
+        result = (*m_active)->isPushing()
+            || (*m_active)->isMovingDown();
+    }
+    return result;
+}
 
 
