@@ -46,6 +46,7 @@
 {
     m_desc = NULL;
     m_restartCounter = 1;
+    m_insideUndo = false;
     m_depth = depth;
     m_newRound = false;
     m_locker = new PhaseLocker();
@@ -66,7 +67,7 @@ Level::~Level()
     own_cleanState();
     delete m_locker;
     //NOTE: m_show must be removed before levelScript
-    // because uses the same script
+    // because it uses the same script
     delete m_show;
     delete m_countdown;
     delete m_loading;
@@ -210,6 +211,29 @@ Level::updateLevel()
     }
 }
 //-----------------------------------------------------------------
+/**
+ * Save state for undo.
+ * Should be called after a player move,
+ * but still before level script update.
+ * @param oldMoves moves before the last move
+ */
+    void
+Level::saveUndo(const std::string &oldMoves)
+{
+    if (oldMoves.empty()) {
+        return;
+    }
+
+    if (m_levelScript->isRoom()) {
+        Room *room = m_levelScript->room();
+        bool forceSave = room->stepCounter()->isDangerousMove();
+
+        std::string forceSaveValue = forceSave ? "true" : "false";
+        m_levelScript->scriptDo("script_saveUndo(\""
+                + oldMoves + "\"," + forceSaveValue + ")");
+    }
+}
+//-----------------------------------------------------------------
 /**
  * Finish complete level.
  * Save solution.
@@ -240,7 +264,15 @@ Level::finishLevel()
 Level::nextPlayerAction()
 {
     if (m_levelScript->isRoom()) {
-         m_levelScript->room()->nextRound(getInput());
+        Room *room = m_levelScript->room();
+        bool wasSolvable = room->isSolvable();
+        std::string oldMoves = room->stepCounter()->getMoves();
+        room->nextRound(getInput());
+
+        unsigned int stepsAfter = room->stepCounter()->getStepCount();
+        if (wasSolvable && stepsAfter != oldMoves.size()) {
+            saveUndo(oldMoves);
+        }
     }
 }
 
@@ -293,7 +325,14 @@ Level::displaySaveStatus()
     void
 Level::loadGame(const std::string &moves)
 {
-    m_loading->loadGame(moves);
+    if (m_insideUndo) {
+        if (m_levelScript->isRoom()) {
+            m_levelScript->room()->setMoves(moves);
+        }
+    }
+    else {
+        m_loading->loadGame(moves);
+    }
 }
 //-----------------------------------------------------------------
 /**
@@ -341,8 +380,8 @@ Level::action_restart()
 {
     own_cleanState();
     m_restartCounter++;
-    //TODO: is ok to run the script on second time?
-    //NOTE: planned show remains after restart
+    //NOTE: The script is just overridden by itself,
+    // so planned shows and undo remain after restart.
     own_initState();
     return true;
 }
@@ -387,6 +426,7 @@ Level::action_load()
         m_restartCounter--;
         action_restart();
         m_levelScript->scriptInclude(file);
+        m_insideUndo = false;
         m_levelScript->scriptDo("script_load()");
     }
     else {
@@ -395,7 +435,22 @@ Level::action_load()
     }
     return true;
 }
+//-----------------------------------------------------------------
+/**
+ * Naive undo by reloading the level.
+ */
+    bool
+Level::action_undo()
+{
+    if (m_levelScript->isRoom()) {
+        m_restartCounter--;
+        action_restart();
 
+        m_insideUndo = true;
+        m_levelScript->scriptDo("script_loadUndo()");
+    }
+    return true;
+}
 //-----------------------------------------------------------------
     void
 Level::switchFish()
