@@ -345,6 +345,12 @@ Room::stepCounter() const
 {
     return m_controls;
 }
+//-----------------------------------------------------------------
+    void
+Room::setMoves(const std::string &moves)
+{
+    m_controls->setMoves(moves);
+}
 //-----------------------------------------------------------------
     void
 Room::checkActive()
