@@ -63,7 +63,7 @@ script_game_setRoomWaves(lua_State *L) throw()
  * int game_addModel(kind, x, y, shape)
  * Return model index.
  *
- *  table = addModel("light", 10, 30, "table.bmp",
+ *  table = addModel("light", 10, 30,
  *  [[
  *  XXXXX
  *  ..X
@@ -583,6 +583,24 @@ script_model_change_turnSide(lua_State *L) throw()
     return 0;
 }
 //-----------------------------------------------------------------
+/**
+ * void model_change_setLocation(model_index, x, y)
+ * Change model position (used to load undo).
+ */
+    int
+script_model_change_setLocation(lua_State *L) throw()
+{
+    BEGIN_NOEXCEPTION;
+    int model_index = luaL_checkint(L, 1);
+    int x = luaL_checkint(L, 2);
+    int y = luaL_checkint(L, 3);
+    Cube *model = getModel(L, model_index);
+    model->rules()->change_setLocation(V2(x, y));
+
+    END_NOEXCEPTION;
+    return 0;
+}
+//-----------------------------------------------------------------
 /**
  * void model_setViewShift(model_index, shift_x, shift_y)
  * Shift view (used for obsolete animation effects).
@@ -634,6 +652,72 @@ script_model_setBusy(lua_State *L) throw()
     END_NOEXCEPTION;
     return 0;
 }
+//-----------------------------------------------------------------
+/**
+ * table model_getExtraParams(model_index)
+ *
+ * Returns extra parameters needed to restore after undo.
+ */
+    int
+script_model_getExtraParams(lua_State *L) throw()
+{
+    BEGIN_NOEXCEPTION;
+    int model_index = luaL_checkint(L, 1);
+    Cube *model = getModel(L, model_index);
+
+    if (model->getOutCapacity() != 0) {
+        lua_newtable(L);
+        lua_pushstring(L, "outDir");
+        lua_pushnumber(L, model->getOutDir());
+        lua_settable(L, -3);
+
+        lua_pushstring(L, "outCapacity");
+        lua_pushnumber(L, model->getOutCapacity());
+        lua_settable(L, -3);
+
+        lua_pushstring(L, "weight");
+        lua_pushnumber(L, model->getWeight());
+        lua_settable(L, -3);
+    }
+    else {
+        lua_pushnil(L);
+    }
+
+    END_NOEXCEPTION;
+    return 1;
+}
+//-----------------------------------------------------------------
+/**
+ * void model_change_setExtraParams(model_index, table)
+ *
+ * Restores extra parameters after undo.
+ */
+    int
+script_model_change_setExtraParams(lua_State *L) throw()
+{
+    BEGIN_NOEXCEPTION;
+    int model_index = luaL_checkint(L, 1);
+
+    if (!lua_isnil(L, 2)) {
+        lua_pushstring(L, "outDir");
+        lua_gettable(L, 2);
+        int outDir = luaL_checkint(L, -1);
+
+        lua_pushstring(L, "outCapacity");
+        lua_gettable(L, 2);
+        int outCapacity = luaL_checkint(L, -1);
+
+        lua_pushstring(L, "weight");
+        lua_gettable(L, 2);
+        int weight = luaL_checkint(L, -1);
+
+        Cube *model = getModel(L, model_index);
+        model->setOutDir((Dir::eDir)outDir, outCapacity, (Cube::eWeight)weight);
+    }
+
+    END_NOEXCEPTION;
+    return 0;
+}
 
 //-----------------------------------------------------------------
 /**
